##### SET THIS DIRECTORY TO THE RESULTS DIRECTORY FIRST #####

setwd("~/Dropbox/Current_work/PF_genome_scale/Figshare/Results")

############################################################


library(ggplot2)


multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  require(grid)

  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                    ncol = cols, nrow = ceiling(numPlots/cols))
  }

 if (numPlots==1) {
    print(plots[[1]])

  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}


### load the strict clustering data and process it for plots ###
r <- read.table("Hackett_analysis/strict_clustering.txt", header=TRUE)
c <- read.table("Hackett_analysis/sampled_weights.txt", header=TRUE)


#now we'll calculate delta AICc and BIC, for plotting
bestBIC = min(r$bestBIC)
bestAICc = min(r$bestAICc)
r$index <- as.factor(r$index)
r$deltaBIC <- NA
r$deltaAICc <- NA
bestBIC.rows <- data.frame()
bestAICc.rows <- data.frame()

count = 0

for(index in levels(r$index)){
	count = count+1
	cat(index, count, "\n")
		
	row.indices = which(r$index==index)
	deltaBIC <- min(r$bestBIC[row.indices])-bestBIC
	deltaAICc <- min(r$bestAICc[row.indices])-bestAICc
	r$deltaBIC[row.indices]=deltaBIC
	r$deltaAICc[row.indices]=deltaAICc
	
	rows = r[row.indices,]
	bestBIC.row <- rows[which(rows$BIC==min(rows$BIC)),]
	bestBIC.rows <- rbind(bestBIC.rows, bestBIC.row)

	bestAICc.row <- rows[which(rows$AICc==min(rows$AICc)),]
	bestAICc.rows <- rbind(bestAICc.rows, bestAICc.row)

	}


# load data from the greedy algorithm on the Hackett_2008 dataset
r$index = as.factor(r$index)
greedyBIC_progress  = read.delim("Hackett_analysis/all_schemes_greedyBIC.txt", sep="", header=TRUE)
greedyAICc_progress = read.delim("Hackett_analysis/all_schemes_greedyAICc.txt", sep="", header=TRUE)




####################### FIGURE 1 ####################################################
###  Box plots of the relaxed  and strict clustering algorithms    		 		 ####
###  compared to the greedy algorithm. 											 ####

bicG  <- min(greedyBIC_progress$BIC)
bicC  <- bestBIC.rows # this is the strict clustering analysis
bic0  <- read.table("Hackett_analysis/relaxed_clustering_BIC_P0.txt")
bic1  <- read.table("Hackett_analysis/relaxed_clustering_BIC_P1.txt")
bic2  <- read.table("Hackett_analysis/relaxed_clustering_BIC_P2.txt")
bic5  <- read.table("Hackett_analysis/relaxed_clustering_BIC_P5.txt")
bic10 <- read.table("Hackett_analysis/relaxed_clustering_BIC_P10.txt")
bic20 <- read.table("Hackett_analysis/relaxed_clustering_BIC_P20.txt")

#now add a column to each before merging
bicC$type <- "Strict clustering"
bic0$type <- "0%"
bic1$type <- "1%"
bic2$type <- "2%"
bic5$type <- "5%"
bic10$type <- "10%"
bic20$type <- "20%"

bicC$order <- 1
bic0$order <- 2
bic1$order <- 3
bic2$order <- 3
bic5$order <- 4
bic10$order <- 5
bic20$order <- 6

BICdata <- rbind(bicC, bic1, bic2, bic5, bic10, bic20)

BICstart <- max(greedyBIC_progress$BIC) # BIC of the starting scheme
BICdata$BICdiff <- BICstart - BICdata$BIC

BICdata$index <- as.numeric(BICdata$index)
BICdata$type <- reorder(BICdata$type, BICdata$order)



aiccG  <- min(greedyAICc_progress$AICc)
aiccC  <- bestAICc.rows
aicc0  <- read.table("Hackett_analysis/relaxed_clustering_AICc_P0.txt")
aicc1  <- read.table("Hackett_analysis/relaxed_clustering_AICc_P1.txt")
aicc2  <- read.table("Hackett_analysis/relaxed_clustering_AICc_P2.txt")
aicc5  <- read.table("Hackett_analysis/relaxed_clustering_AICc_P5.txt")
aicc10 <- read.table("Hackett_analysis/relaxed_clustering_AICc_P10.txt")
aicc20 <- read.table("Hackett_analysis/relaxed_clustering_AICc_P20.txt")

#now add a column to each before merging
aiccC$type <- "Strict clustering"
aicc0$type <- "0%"
aicc1$type <- "1%"
aicc2$type <- "2%"
aicc5$type <- "5%"
aicc10$type <- "10%"
aicc20$type <- "20%"

aiccC$order <- 1
aicc0$order <- 2
aicc1$order <- 3
aicc2$order <- 3
aicc5$order <- 4
aicc10$order <- 5
aicc20$order <- 6

AICcdata <- rbind(aiccC, aicc1, aicc2, aicc5, aicc10, aicc20)


AICcstart <- max(greedyAICc_progress$AICc) # AICc of the starting scheme
AICcdata$AICcdiff <- AICcstart - AICcdata$AICc
AICcdata$index <- as.numeric(AICcdata$index)
AICcdata$type <- reorder(AICcdata$type, AICcdata$order)


pBIC <- ggplot(BICdata, aes(factor(type), y=BIC))
pBIC <- pBIC + geom_boxplot(width=0.5) + geom_hline(aes(yintercept=bicG), linetype='dashed') + 
		xlab("Algorithm") + ylab("BIC score")
		
		
pAICc <- ggplot(AICcdata, aes(factor(type), y=AICc))
pAICc <- pAICc + geom_boxplot(width=0.5) + geom_hline(aes(yintercept=aiccG), linetype='dashed') + 
		 xlab("Algorithm") + ylab("AICc score")

multiplot(pBIC, pAICc, cols=1)

dev.copy2pdf(file="Figure1_draft.pdf")
dev.off()


####################### FIGURE 2 ###########################################
### Scatterplots of the different weights versus the BIC/AICc results   ####
### For the strict clustering algorithms  			 					####
### Red dots for the best score, to highlight best weights  			####


b1 <- ggplot(bestBIC.rows, aes(rates, deltaBIC)) + geom_point(alpha=0.25) + geom_point(data = bestBIC.rows[which(bestBIC.rows$deltaBIC==0),], aes(rates, deltaBIC), colour='red', size=3) + theme(legend.position="none") + xlab("subset rate")
b2 <- ggplot(bestBIC.rows, aes(freqs, deltaBIC)) + geom_point(alpha=0.25) + geom_point(data = bestBIC.rows[which(bestBIC.rows$deltaBIC==0),], aes(freqs, deltaBIC), colour='red', size=3) + theme(legend.position="none") + xlab("base frequencies")
b3 <- ggplot(bestBIC.rows, aes(model, deltaBIC)) + geom_point(alpha=0.25) + geom_point(data = bestBIC.rows[which(bestBIC.rows$deltaBIC==0),], aes(model, deltaBIC), colour='red', size=3) + theme(legend.position="none") + xlab("GTR model parameters")
b4 <- ggplot(bestBIC.rows, aes(alpha, deltaBIC)) + geom_point(alpha=0.25) + geom_point(data = bestBIC.rows[which(bestBIC.rows$deltaBIC==0),], aes(alpha, deltaBIC), colour='red', size=3) + theme(legend.position="none") + xlab("alpha parameter")

a1 <- ggplot(bestAICc.rows, aes(rates, deltaAICc)) + geom_point(alpha=0.25) + geom_point(data = bestAICc.rows[which(bestAICc.rows$deltaAICc==0),], aes(rates, deltaAICc), colour='red', size=3) + theme(legend.position="none") + xlab("subset rate")
a2 <- ggplot(bestAICc.rows, aes(freqs, deltaAICc)) + geom_point(alpha=0.25) + geom_point(data = bestAICc.rows[which(bestAICc.rows$deltaAICc==0),], aes(freqs, deltaAICc), colour='red', size=3) + theme(legend.position="none") + xlab("base frequencies")
a3 <- ggplot(bestAICc.rows, aes(model, deltaAICc)) + geom_point(alpha=0.25) + geom_point(data = bestAICc.rows[which(bestAICc.rows$deltaAICc==0),], aes(model, deltaAICc), colour='red', size=3) + theme(legend.position="none") + xlab("GTR model parameters")
a4 <- ggplot(bestAICc.rows, aes(alpha, deltaAICc)) + geom_point(alpha=0.25) + geom_point(data = bestAICc.rows[which(bestAICc.rows$deltaAICc==0),], aes(alpha, deltaAICc), colour='red', size=3) + theme(legend.position="none") + xlab("alpha parameter")

quartz(width=10, height=4)
multiplot(b1, a1, b2, a2, b3, a3, b4, a4, cols=4)
dev.copy2pdf(file="Figure2_draft.pdf")
dev.off()



####################### FIGURE 3 ##################################################
#### Comparison of scores of the three algorithms on all datasets from table 1 ####

d <- read.csv(file="Table1_results.csv", header=TRUE)

dR = d[which(d$Algorithm=='rcluster'),]
dH = d[which(d$Algorithm=='hcluster'),]
dG = d[which(d$Algorithm=='greedy'),]

dR$Scorediff <- dR$Score - dG$Score
dH$Scorediff <- dH$Score - dG$Score
dG$Scorediff <- dG$Score - dG$Score

dR$Timeprop <- dG$Time - dR$Time
dH$Timeprop <- dG$Time - dH$Time
dG$Timeprop <- dG$Time - dG$Time

dR$ScorePC <- dR$ScoreImprovement*100 / dG$ScoreImprovement
dH$ScorePC <- dH$ScoreImprovement*100 / dG$ScoreImprovement
dG$ScorePC <- dG$ScoreImprovement*100 / dG$ScoreImprovement

dR$TimePC <- dR$Time*100 / dG$Time
dH$TimePC <- dH$Time*100 / dG$Time
dG$TimePC <- dG$Time*100 / dG$Time

dR$label = "Relaxed clustering"
dH$label = "Strict clustering"
dG$label = "Greedy"

d2 <- rbind(dR, dH, dG)

# here we plot the performance of the two algorithms relative to the greedy algorithm
quartz(width=8, height=4)
fig3 <- ggplot(d2, aes(x = label, y = ScorePC, group=Dataset)) + 
	    geom_line(alpha=0.5) + 
        geom_point(size=3, colour='white') + 
        geom_point(alpha=0.5, size=3) + 
        facet_wrap(~Scoretype) + 
        scale_x_discrete("Algorithm used for analysis") + 
        scale_y_continuous("Percentage improvement in\nAICc or BIC relative to greedy algorithm")
fig3

dev.copy2pdf(file="Figure3_draft.pdf")
dev.off()



####################### FIGURE 3 ##################################################
#### Comparison of times of the three algorithms on all datasets from table 1  ####

quartz(width=8, height=4)
fig4 <- ggplot(d2, aes(x = label, y = TimePC, group=Dataset)) + 
	    geom_line(alpha=0.5) + 
        geom_point(size=3, colour='white') + 
        geom_point(alpha=0.5, size=3) + 
        facet_wrap(~Scoretype) + 
        scale_x_discrete("Algorithm used for analysis") + 
        scale_y_continuous("Percentage of time taken\nrelative to greedy algorithm") 
fig4

dev.copy2pdf(file="Figure4_draft.pdf")
dev.off()












